require "defines"
require "story"

-- The story contents
storytable =
{{
  {
    action =
    function()
      game.showmessagedialog{text = {"msg-introduction"}}
      game.showmessagedialog{text = {"msg-ask-chest"}}
    end
  },
  {
    action =
    function()
      local player = game.getplayer(1)
      if player ~= nil then
        player.gui.top.add{type = "button", name="button_use_chest", caption={"button-use-chest"}}
        player.gui.top.add{type = "button", name="button_no_chest", caption={"button-no-chest"}}
      end
    end
  },
  {
    condition =
    function(event)
      if event.name == defines.events.onguiclick then
        local player = game.getplayer(event.playerindex)
        if event.element.name == "button_use_chest" then
          buildstartingchest(player)
        elseif event.element.name ~= "button_no_chest" then
          return false
        end
        player.gui.top.button_use_chest.destroy()
        player.gui.top.button_no_chest.destroy()
        return true
      end
      return false
    end,
    action = function()
      game.showmessagedialog{text = {"msg-ask-technologies"}}
    end
  },
  {
    action =
    function()
      local player = game.getplayer(1)
      if player ~= nil then
        player.gui.top.add{type = "button", name = "button_technologies_researched", caption = {"button-technologies-researched"}}
        player.gui.top.add{type = "button", name = "button_technologies_normal", caption = {"button-technologies-normal"}}
      end
    end
  },
  {
    condition =
    function(event)
      if event.name == defines.events.onguiclick then
        local player = game.getplayer(event.playerindex)
        if event.element.name == "button_technologies_researched" then
          player.force.researchalltechnologies()
        elseif event.element.name ~= "button_technologies_normal" then
          return false
        end
        player.gui.top.button_technologies_researched.destroy()
        player.gui.top.button_technologies_normal.destroy()
        return true
      end
      return false
    end,
    action = function()
      action = function() game.showmessagedialog{text = {"msg-start-play"}} end
    end
  },
  -- start the game
  {
    -- After this point we don't need events any more, so we just disable them all
    action =
    function()
      game.onevent(defines.events, nil)
    end
  },
  {
    condition = function() return false end,
    action = function() end
  }
}}

mainstory = story.Story:new(storytable)

game.onsave(function()
  glob.storydata = mainstory:export()
end)

game.onload(function()
  mainstory:import(glob.storydata)
end)

-- We need to update the story after every event except onplayercreated, so the order
-- of the tw following definitions is important.

game.onevent(defines.events, function(event)
  mainstory:update(event, "")
end)

game.onevent(defines.events.onplayercreated, function(event)
  local player = game.getplayer(event.playerindex)
  local character = player.character
  player.character = nil
  if character then character.destroy() end
end)

function buildstartingchest(player)
  local chestposition = game.findnoncollidingposition("steel-chest", player.position, 0, 0.1)
  local chest = game.createentity{name="steel-chest", position=chestposition, force = game.forces.player}
  chest.insert({name="wood", count="50"})
  chest.insert({name="coal", count="100"})
  chest.insert({name="stone", count="100"})
  chest.insert({name="iron-plate", count="400"})
  chest.insert({name="copper-plate", count="400"})
  chest.insert({name="steel-plate", count="100"})
  chest.insert({name="iron-gear-wheel", count="200"})
  chest.insert({name="electronic-circuit", count="200"})
  chest.insert({name="advanced-circuit", count="200"})
  chest.insert({name="offshore-pump", count="20"})
  chest.insert({name="pipe", count="50"})
  chest.insert({name="boiler", count="50"})
  chest.insert({name="basic-mining-drill", count="50"})
  chest.insert({name="steam-engine", count="10"})
  chest.insert({name="stone-furnace", count="50"})
  chest.insert({name="basic-transport-belt", count="100"})
  chest.insert({name="fast-transport-belt", count="50"})
  chest.insert({name="express-transport-belt", count="50"})
  chest.insert({name="basic-inserter", count="50"})
  chest.insert({name="fast-inserter", count="50"})
  chest.insert({name="long-handed-inserter", count="50"})
  chest.insert({name="smart-inserter", count="50"})
  chest.insert({name="small-electric-pole", count="50"})
  chest.insert({name="assembling-machine-1", count="50"})
  chest.insert({name="assembling-machine-2", count="50"})
  chest.insert({name="straight-rail", count="150"})
  chest.insert({name="curved-rail", count="50"})
  chest.insert({name="train-stop", count="10"})
  chest.insert({name="rail-signal", count="50"})
  chest.insert({name="diesel-locomotive", count="5"})
  chest.insert({name="cargo-wagon", count="5"})
end
